import { Guess } from "./guess.ts";
import { clang_format } from "./utils.ts";
import { source_distance } from "./source_distance.ts";

export class ConfigFile {
	properties;

	constructor(properties: [string, string][]) {
		this.properties = properties;
	}

	static from_guesses(guesses: Guess[]) {
		return new ConfigFile(guesses.map((g) => [g.property.name, g.value]));
	}

	async calculate_certainty(source: string) {
		const formatted = await clang_format(source, ...this.properties);
		const distance = source_distance(source, formatted);
		return (source.length - distance) / source.length;
	}

	serialize() {
		return this.properties.map(([name, value]) => `${name}: ${value}`).join("\n");
	}
}
