import { Guess } from "./guess.ts";
import { indexed } from "./utils.ts";

export class Property {
	name;
	possible_values;

	constructor(name: string, possible_values: string[]) {
		this.name = name;
		this.possible_values = possible_values;
	}

	async guess_from_source(source: string) {
		const score_promises = [] as Promise<[number, Guess, number]>[];
		for (const [index, value] of indexed(this.possible_values)) {
			const guess = new Guess(this, value);
			score_promises.push((async () => [index, guess, await guess.test(source)])());
		}
		const scored_guesses = await Promise.all(score_promises);
		if (all_equals(scored_guesses.map(([_, __, s]) => s))) return null;
		// note : Sorts by score (highest first) then by index (lowest first).
		const sorted_gueses = scored_guesses.toSorted(([ia, _, a], [ib, __, b]) => (b - a) == 0 ? (ia - ib) : (b - a));
		const [_, guess, __] = sorted_gueses[0];
		return guess;
	}
}

function all_equals<T>(arr: T[]) {
	if (arr.length === 0) return true;
	const [first] = arr;
	for (const item of arr) if (item !== first) return false;
	return true;
}
