import { assertEquals } from "https://deno.land/std@0.220.1/assert/mod.ts";
import { range } from "./utils.ts";

export function source_distance(a: string, b: string) {
	// return source_distance_impl(a, b);
	return source_distance_wrap(a, b);
}

import { distance } from "https://deno.land/x/fastest_levenshtein@1.0.10/mod.ts";
export function source_distance_wrap(a: string, b: string) {
	a = unify_indents(a);
	b = unify_indents(b);
	return distance(a, b);
}

export function source_distance_impl(a: string, b: string) {
	a = unify_indents(a);
	b = unify_indents(b);
	const [rest_a, rest_b] = remove_common_fragments(a, b, 6);
	const len_a = rest_a.replaceAll(sep_seq, "").length;
	const len_b = rest_b.replaceAll(sep_seq, "").length;
	return len_a + len_b;
}

const sep_seq = "🥖";

function unify_indents(text: string) {
	const after = text.replaceAll("\t", " ").replaceAll("  ", " ");
	if (after === text) return text;
	else return unify_indents(after);
}

function remove_common_fragments(a: string, b: string, min_frag_size: number) {
	const result = find_fragment_location(a, b, min_frag_size);
	if (result === null) return [a, b];
	const [ia, ib, size] = result;
	const new_a = remove_fragment(a, ia, size);
	const new_b = remove_fragment(b, ib, size);
	return remove_common_fragments(new_a, new_b, min_frag_size);
}

function find_fragment_location(a: string, b: string, min_frag_size: number) {
	for (const start_a of range(0, a.length)) {
		const slice_a = a.slice(start_a);
		for (const start_b of range(0, b.length)) {
			const slice_b = b.slice(start_b);
			const common_count = common_length(slice_a, slice_b);
			if (common_count < min_frag_size) continue;
			return [start_a, start_b, common_count] as const;
		}
	}
	return null;
}

function common_length(a: string, b: string) {
	let result = 0;
	for (const index of range(0, Math.min(a.length, b.length))) {
		if (a[index] != b[index]) break;
		result += 1;
	}
	return result;
}

function remove_fragment(text: string, start: number, size: number) {
	const prefix = text.slice(0, start);
	const suffix = text.slice(start + size);
	return prefix + sep_seq + suffix;
}

Deno.test("test_source_distance", () => {
	const source_a = `
int main() {
	if (a) b;
}`;
	const source_b = `
int main()
{
	if (a)
	{
		b;
	}
}`;
	assertEquals(source_distance(source_a, source_b), 19);
});
